@@ -1,4 +1,44 @@
-FRBpcaMM <- function(Y, R=999, conf=0.95, control=MMcontrol(...), ...)
+FRBpcaMM <- function(Y,...) UseMethod("FRBpcaMM")
+
+
+FRBpcaMM.formula <- function (formula, data = NULL, ...) 
+{
+.check_vars_numeric <- function (mf) 
+{
+    mt <- attr(mf, "terms")
+    mterms <- attr(mt, "factors")
+    mterms <- rownames(mterms)[apply(mterms, 1, any)]
+    any(sapply(mterms, function(x) is.factor(mf[, x]) || !is.numeric(mf[, 
+        x])))
+}
+#--------------------------------------------------------------------------
+# Main function
+
+    cl <- match.call()
+    mt <- terms(formula, data = data)
+    if (attr(mt, "response") > 0L) 
+        stop("response not allowed in formula")
+    mf <- match.call(expand.dots = FALSE)
+    mf$... <- NULL
+    mf[[1L]] <- as.name("model.frame")
+    mf <- eval.parent(mf)
+    if (.check_vars_numeric(mf)) 
+        stop("PCA applies only to numerical variables")
+    na.act <- attr(mf, "na.action")
+    mt <- attr(mf, "terms")
+    attr(mt, "intercept") <- 0L
+    x <- model.matrix(mt, mf)
+    res <- FRBpcaMM.default(x, ...)
+  	cl <- match.call()
+    cl[[1]] <- as.name("FRBpcaMM")
+    res$call <- cl
+    if (!is.null(na.act)) res$na.action <- na.act
+    return(res)
+}
+
+
+
+FRBpcaMM.default <- function(Y, R=999, conf=0.95, control=MMcontrol(...),na.action=na.omit, ...)
 {
 # performs PCA based on the multivariate MM estimate of shape, with
 # fast and robust bootstrap
@@ -80,8 +120,15 @@ return(rec)
 # --------------------------------------------------------------------
 
 Y <- as.matrix(Y)
+Y=na.action(Y)
+
+
 n <- nrow(Y)
 q <- ncol(Y)
+if (q <= 1L) stop("at least two variables needed for PCA")
+if (n < q) stop("For PCA the number of observations cannot be smaller than the 
+number of variables")
+
 dimens <- q + q*q
 
 # compute MM-estimates of location and shape
@@ -111,7 +158,6 @@ for (k in 1:(q-1)) {
 # compute bootstrapped MM-estimates of location and shape
 bootres <- MMboot_loccov(Y, R, ests=MMests)
 
-
 ##################################################################################
 # now take the bootstrapped shape estimates and compute their eigenvalues/vectors
 
@@ -122,7 +168,6 @@ bootangles <- matrix(0,q,R)
 bootsampleOK <- rep(1,R)
 
 for (r in 1:R) {
-
     correctedMMGammast <- reconvec(bootres$centered[(q+1):dimens,r],q) + MMGamma
     eigresGammast <- eigen(correctedMMGammast)
     IXGammast <- order(eigresGammast$values, decreasing=TRUE)    
@@ -143,7 +188,7 @@ for (r in 1:R) {
 
     Svecs <- eigenvectorsGammast
     for (k in 1:q) {
-        bootangles[k,r] <- acos(abs(t(Svecs[,k]) %*% MMeigvecs[,k]/sqrt(t(Svecs[,k]) %*% Svecs[,k])/sqrt(t(MMeigvecs[,k])%*%MMeigvecs[,k])))
+        bootangles[k,r] <- acos(min(abs(t(Svecs[,k]) %*% MMeigvecs[,k]/sqrt(t(Svecs[,k]) %*% Svecs[,k])/sqrt(t(MMeigvecs[,k])%*%MMeigvecs[,k])),1))
     }
     booteigvalsGamma[,r] <- eigenvaluesGammast
     booteigvecs[,r] <- vecop(eigenvectorsGammast)
@@ -161,7 +206,7 @@ if (nfailed > 0.75*R) {
 
 bootangles <- bootangles[,bootindicesOK]  
 booteigvalsGamma <- booteigvalsGamma[,bootindicesOK]
-bootpercvars <- bootpercvars[,bootindicesOK]
+bootpercvars <- bootpercvars[,bootindicesOK,drop=FALSE]
 booteigvecs <- booteigvecs[,bootindicesOK]
   
 avgangle <- apply(bootangles,1,mean)
